@@ -64,6 +64,19 @@ NULL
     }
 
   ))
+
+  # provide a common base S3 class for tensors
+  reticulate::register_class_filter(function(classes) {
+    if (any(c("tensorflow.python.ops.variables.Variable",
+              "tensorflow.python.framework.ops.Tensor")
+            %in%
+            classes)) {
+      c("tensorflow.tensor", classes)
+    } else {
+      classes
+    }
+  })
+
 }
 
 
